package za.org.treehouse.hypoalarm;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Build;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.util.Log;

public class AlarmNotify extends Service {
    public  static final int notifyID = 1;
    public volatile boolean notificationRunning = false;

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public void onDestroy() {
        // If the notification is cancelled, stop updating.
        notificationRunning = false;
        // Remove the notification in the notification bar
        NotificationManager nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
        nm.cancel(notifyID);
        Log.d("AlarmNotify", "Notification stopped.");
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        final int UPDATE_INTERVAL = 10*1000; // Timer is updated six times a minute
        SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(this);

        Log.d("AlarmNotify", "Notification started.");

        //final String phoneNumber = sharedPref.getString(getString(R.string.PhoneNumberPref), null);
        final int gracePeriod = sharedPref.getInt(getString(R.string.GracePeriodPref), 60);

        Bitmap bm = BitmapFactory.decodeResource(getResources(), R.drawable.ic_launcher_grey);
        final NotificationManager nm = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
        if (Build.VERSION.SDK_INT >= 11) {
            final Notification.Builder notification = new Notification.Builder(this)
                    .setContentTitle(getString(R.string.app_name))
                    .setContentText(String.format(getString(R.string.notificationText), MainActivity.MinutesToGracePeriodStr(gracePeriod)))
                    .setSmallIcon(R.drawable.alarm_notification)
                    .setLargeIcon(bm)
                    .setOnlyAlertOnce(true)
                    .setAutoCancel(false)
                    .setPriority(Notification.PRIORITY_HIGH);


// TODO if alarm alert is snoozing and we cancel, cancel the snooze.
            // Set up dismiss action
            Intent cancellerIntent = new Intent(getBaseContext(), CancelGraceReceiver.class);
            PendingIntent cancellerPendingIntent = PendingIntent.getBroadcast(getBaseContext(), MainActivity.CANCEL_GRACE_REQUEST, cancellerIntent, PendingIntent.FLAG_CANCEL_CURRENT);

            // Cancel the grace period if the user clears the notification
            notification.setDeleteIntent(cancellerPendingIntent);
            // Allow the user to cancel by clicking a "Cancel" button
            notification.addAction(android.R.drawable.ic_menu_close_clear_cancel, getString(R.string.notificationCancellation), cancellerPendingIntent);
            // Allow the user to cancel by selecting the ContentText or ContentTitle
            notification.setContentIntent(cancellerPendingIntent);

            /**
             * TODO load alert activity (without sound or vibration) on select?
             * TODO This would allow the user to test competence
             * something like:
             Intent alarmAlertIntent = new Intent(this, AlarmAlertActivity.class);
             PendingIntent alarmPendingIntent = PendingIntent.getBroadcast(this, 0, alarmAlertIntent, 0);
             notification.setContentIntent(alarmPen`dingIntent);
             */


            nm.cancel(notifyID);
            nm.notify(notifyID, notification.build());

            new Thread(new Runnable() {
                @Override
                public void run() {
                    notificationRunning = true;
                    int max = 1000;
                    /* TODO check that graceEndTime is always set.
                    if (AlarmReceiver.graceEndTime == 0) {
                        AlarmReceiver.graceEndTime = System.currentTimeMillis() + (gracePeriod * 60 * 1000);
                    }*/
                    // Count in milliseconds for greater progress resolution
                    int milliSecondsLeft = (int) ((AlarmReceiver.graceEndTime - System.currentTimeMillis()));
                    int gracePeriodMilliSeconds = gracePeriod * 60 * 1000;
                    int progress = ((gracePeriodMilliSeconds - milliSecondsLeft) * max) / gracePeriodMilliSeconds;

                    while (progress < max) {
                        // Stop the thread if the notification is cancelled elsewhere
                        if (!notificationRunning) {
                            return;
                        }
                        int minutesLeft = (milliSecondsLeft / 1000) / 60;
                        if (Build.VERSION.SDK_INT >= 11) {
                            notification.setContentText(String.format(getString(R.string.notificationText), MainActivity.MinutesToGracePeriodStr(minutesLeft)));
                            notification.setProgress(max, progress, false);
                            // Update the notification
                            nm.notify(notifyID, notification.build());
                        }
                        // Prepare secondsLeft and progress for the next loop
                        milliSecondsLeft = milliSecondsLeft - UPDATE_INTERVAL;
                        // Multiply each int by 1000 for greater progress resolution
                        progress = ((gracePeriodMilliSeconds - milliSecondsLeft) * max) / gracePeriodMilliSeconds;
                        //Log.d("AlarmNotify", "milliSecondsLeft is " + milliSecondsLeft + " and progress is " + progress + " (gracePeriodMilliSeconds is " + gracePeriodMilliSeconds + ")");

                        // Sleep until we need to update again
                        try {
                            Thread.sleep(UPDATE_INTERVAL);
                        } catch (InterruptedException e) {
                            Log.d("AlarmNotify", "sleep failure");
                        }
                    }
                    stopSelf(); // stop notification service
                }
            }).start();
        }
		return super.onStartCommand(intent, flags, startId);
    }
}
